/*
 GBA 2MB, 512Kbit Flash Save, Solar + Rumble Flash Cart Tech Demo
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 10/11/2020
 Last Modified: 10/11/2020
 
 */

#include <gba_console.h>
#include <gba_video.h>
#include <gba_interrupt.h>
#include <gba_systemcalls.h>
#include <gba_input.h>
#include <gba_timers.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

u16 kpInput = 0;
u16 kpRead = 0;

#define GBAROM ((u16*)0x08000000)
#define CART_SRAM 0xe000000

// Write a byte
void sendData (u32 address, u32 data) {
   asm volatile(
	 "mov    r3,%0\n\
		mov    r4,%1\n\
		strh   r4,[r3]\n\
      " :
	  
      :  // No output
	  "r" (address), "r" (data) :  // Define the routine inputs (%0,%1,%2).
      "r3", "r4" );   
}


int main(void) {
	irqInit();
	irqEnable(IRQ_VBLANK);
	//consoleDemoInit();
	
	consoleInit(	0,		// charbase
					4,		// mapbase
					0,		// background number
					NULL,	// font
					0, 		// font size
					15		// 16 color palette
	);
	
	
	// set the screen colors, color 0 is the background color
	// the foreground color is index 1 of the selected 16 color palette
	BG_COLORS[0]=RGB8(64,64,64);
	BG_COLORS[241]=RGB5(31,31,31);
	SetMode(MODE_0 | BG0_ON);
	
	iprintf("\x1b[0;8HinsideGadgets");
	iprintf("\x1b[1;1HSolar+Rumble+Flash Demo v1.0");
	iprintf("\x1b[3;0HA = Rumble");
	iprintf("\x1b[4;0HUp = Write 0x19 to Save flash");
	iprintf("\x1b[5;0HSel = Sector erase Save flash");
	
	sendData(0x80000C8, 0x01); // I/O Port Control
	sendData(0x80000C6, 0x08); // I/O Port Direction
	
	
	while (1) {
		kpRead = REG_KEYINPUT ^ 0x3FF;
		
		if (kpRead & KEY_A) {
			sendData(0x80000C4, 0x08); // Set motor on
			for (u32 volatile x = 0; x < 50000; x++) {} // Delay a bit
			sendData(0x80000C4, 0x00); // Set motor off
		}
		
		else if (kpRead & KEY_UP) {
			u8 writeValue = 0x19;
			
			// Write a single byte
			*(volatile u8 *)(CART_SRAM+0x5555) = 0xAA;
			*(volatile u8 *)(CART_SRAM+0x2AAA) = 0x55;
			*(volatile u8 *)(CART_SRAM+0x5555) = 0xA0;
			*(volatile u8 *)(CART_SRAM) = writeValue;
			for (u32 volatile x = 0; x < 100; x++) {} // Delay a bit
			
			iprintf("\x1b[9;0HWriting byte 0x19             ");
			
			// Check if the data was written successfully
			u8 readValue = *(volatile u8 *)(CART_SRAM);
			while (readValue != writeValue) {
				for (u32 volatile x = 0; x < 100; x++) {}
				readValue = *(volatile u8 *)(CART_SRAM);
				printf("%X", readValue);
			}
			
			// Wait for key to be let go
			while (kpRead & KEY_UP) {
				kpRead = REG_KEYINPUT ^ 0x3FF;
				for (u32 volatile x = 0; x < 10000; x++) {}
			}
		}
		else if (kpRead & KEY_SELECT) {
			// Erase the first sector
			*(volatile u8 *)(CART_SRAM+0x5555) = 0xAA;
			*(volatile u8 *)(CART_SRAM+0x2AAA) = 0x55;
			*(volatile u8 *)(CART_SRAM+0x5555) = 0x80;
			*(volatile u8 *)(CART_SRAM+0x5555) = 0xAA;
			*(volatile u8 *)(CART_SRAM+0x2AAA) = 0x55;
			*(volatile u8 *)(CART_SRAM) = 0x30;
			for (u32 volatile x = 0; x < 100; x++) {} // Delay a bit
			
			iprintf("\x1b[9;0HErasing sector                ");
			
			// Wait for the first byte to read back as 0xFF
			u8 readValue = *(volatile u8 *)(CART_SRAM);
			while (readValue != 0xFF) {
				for (u32 volatile x = 0; x < 10000; x++) {}
				readValue = *(volatile u8 *)(CART_SRAM);
				printf("%X", readValue);
			}
			
			// Wait for key to be let go
			while (kpRead & KEY_SELECT) {
				kpRead = REG_KEYINPUT ^ 0x3FF;
				for (u32 volatile x = 0; x < 10000; x++) {}
			}
		}
		
		iprintf("\x1b[9;0HFlash data:               ");
		iprintf("\x1b[10;0H");
		for (u32 volatile x = 0; x < 10; x++) {
			u8 i = *(u8 *)(CART_SRAM+x);
			printf("%X ", i);
		}
		
		// Request solar bit, we recommend you run this in WRAM if possible
		sendData(0x8000000, 0x0099); // Activate
		u8 solarbuffer[1];
		memcpy(solarbuffer, (void*)((int)GBAROM), 1); // Read 0x8000000
		for (u32 volatile x = 0; x < 10; x++) {} // Delay a tiny bit
		sendData(0x8000000, 0x0000); // Deactivate		
		iprintf("\x1b[7;0HSolar sensor = %X", solarbuffer[0]);
		
		VBlankIntrWait();
	}
}
